package wechat

import (
	"bytes"
	"context"
	"crypto/rsa"
	"dc_golang_server_1/exception"
	baseModel "dc_golang_server_1/model/base"
	baseService "dc_golang_server_1/service/base"
	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/core/auth/verifiers"
	"github.com/wechatpay-apiv3/wechatpay-go/core/downloader"
	"github.com/wechatpay-apiv3/wechatpay-go/core/notify"
	"github.com/wechatpay-apiv3/wechatpay-go/core/option"
	"github.com/wechatpay-apiv3/wechatpay-go/utils"
	"log"
)

type WxClient struct {
	Client     *core.Client
	MchConfig  baseModel.WechatMchConfig
	PrivateKey *rsa.PrivateKey
}

func GetHandler(mchID string) *notify.Handler {
	wechatMchConfig, err := baseService.FindWechatMchConfig(mchID)
	if err != nil {
		exception.ThrowsErr(err)
	}
	if wechatMchConfig.MchId == nil {
		exception.ThrowsErrS("[" + mchID + "]对应配置不存在，请检查")
	}
	// 获取平台证书访问器
	certVisitor := downloader.MgrInstance().GetCertificateVisitor(mchID)
	handler := notify.NewNotifyHandler(*wechatMchConfig.ApiV3Key, verifiers.NewSHA256WithRSAVerifier(certVisitor))
	return handler
}

func BuildClient(mchID string) WxClient {
	wechatMchConfig, err := baseService.FindWechatMchConfig(mchID)
	if err != nil {
		exception.ThrowsErr(err)
	}
	if wechatMchConfig.MchId == nil {
		exception.ThrowsErrS("[" + mchID + "]对应配置不存在，请检查")
	}
	client, privateKey := initClientByMchID(mchID, *wechatMchConfig.MchSerialNo, *wechatMchConfig.ApiV3Key, *wechatMchConfig.PrivateKey)
	return WxClient{client, wechatMchConfig, privateKey}
}

func initClientByMchID(mchID string, mchCertificateSerialNumber string, mchAPIv3Key string, mchPrivateKey string) (*core.Client, *rsa.PrivateKey) {
	// 使用 utils 提供的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
	//privateKey, err := utils.LoadPrivateKeyWithPath(PKeyPath)
	privateKey := loadPrivateKey(mchPrivateKey)
	ctx := context.Background()
	// 使用商户私钥等初始化 client，并使它具有自动定时获取微信支付平台证书的能力
	opts := []core.ClientOption{
		option.WithWechatPayAutoAuthCipher(mchID, mchCertificateSerialNumber, privateKey, mchAPIv3Key),
	}
	wechatClient, err := core.NewClient(ctx, opts...)
	if err != nil {
		exception.ThrowsErr(err)
	}
	return wechatClient, privateKey
}

func loadPrivateKey(privateKeyStr string) *rsa.PrivateKey {
	privateKey, err := utils.LoadPrivateKey(privateKeyStr)
	if err != nil {
		exception.ThrowsErr(err)
	}
	return privateKey
}

func SignPay(appId string, timeStamp string, nonceStr string, packageInfo string, privateKey *rsa.PrivateKey) string {
	var data bytes.Buffer
	data.WriteString(appId)
	data.WriteString("\n")
	data.WriteString(timeStamp)
	data.WriteString("\n")
	data.WriteString(nonceStr)
	data.WriteString("\n")
	data.WriteString(packageInfo)
	data.WriteString("\n")
	log.Println("需要签名的信息：" + data.String())
	signStr, err := utils.SignSHA256WithRSA(data.String(), privateKey)
	if err != nil {
		exception.ThrowsErr(err)
	}
	return signStr
}
